/*
 * Common font-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __FONT_H__
#define __FONT_H__

#include <stdint.h>



/* Character definition structure. */

typedef struct
{
    uint8_t width;
    uint8_t height;
    uint8_t data[];

} char_definition_t;

typedef struct
{
    uint32_t base;
    uint32_t limit;
    uint32_t table[];

} font_range_t;

/* Font configuration type. */

typedef struct
{
    char_definition_t *chars;
    font_range_t *range;

} font_config_t;



/* Initialise a font.

   Font(<name>, uint8_t *chars, font_range_t *range)
*/

#define Font(NAME, CHARS, RANGE)   \
    font_config_t NAME = {                      \
        .chars = (char_definition_t *) (CHARS), \
        .range = RANGE};

/* Access functions. */

char_definition_t *get_char_definition(font_config_t *cfg, char c);

int write_char(display_config_t *cfg, font_config_t *fcfg, char c,
               int x, int y, uint8_t colour);

void write_string(display_config_t *cfg, font_config_t *fcfg, const char *s,
                  int *x, int *y, uint8_t colour);

#endif /* __FONT_H__ */
